#ifndef BTREE_MEM_H_
#define BTREE_MEM_H_


// #define DEBUG


struct btree_mem_node;
struct btree_mem;

// Constructor and destructor
// When calling btree_mem_new, the min_degree should be the minimum degree of the B-Tree (using Cormen et al's terminology)
// and the cmp function should take pointers to two keys a and b and return -1 if a<b, 0 if a==b, and 1 if a>b
struct btree_mem * btree_mem_new          (int min_degree, int (*cmp)(void *, void *));
void               btree_mem_delete       (struct btree_mem * bt);

void *             btree_mem_insert       (struct btree_mem * bt, void * key, void * data);
void *             btree_mem_find         (struct btree_mem * bt, void * key);
void *             btree_mem_remove       (struct btree_mem * bt, void * key, void ** key_value, void ** data_value);

// Walk the tree in key order, starting with the lowest, and apply the function f to each one.  The argumemnts to f are
// int depth in the tree, the pointer to the key, and the pointer to the data. Note that the key cannot be modified,
// but the data can. Be careful with that.  You are given a pointer to the data, not a pointer to the pointer.  
// In other words, as long as your manipulation of the data does not change it's address, or the size of the data
// memory block, then you should be safe.  If you need to actually replace the data entirely, though, you should 
// remove and insert.
void               btree_mem_walk_inorder (struct btree_mem * bt, void (*f)(int, void const * const, void * const));

#ifdef DEBUG
// btree_mem_dump takes a pointer to a function that prints a represenation of the keya nd data to the stdout.
void               btree_mem_dump         (struct btree_mem * bt, void (*f)(int, void const * const, void * const));
#endif

#endif
